

class AuditAction:
    """
    A class for conveniently specifying action filters in the audit log.

    This class contains a set of constants representing various types of actions 
    in the audit log. It is used for filtering records when calling `client.get_audit_logs`.

    Usage examples:
        logs = client.get_audit_logs(action_type=AuditAction.CreateChannel)
        logs = client.get_audit_logs(action_type=AuditAction.DeleteMember)
    
    """
    AllActions = None
    CreateChannel: int = 10
    ServerUpdate: int = 1
    ChannelUpdate: int = 11
    DeleteChannel: int = 12
    CreateChannelRights: int = 13
    ChangeChannelRights: int = 14
    DeleteChannelRights: int = 15
    DeleteMember: int = 20

    #there are many more types of actions in the audit log... I'll add them all a bit later...


from . import UserProfile, Message

class Channels:
	def __init__(self, data: list = []):
		self.data = data
		self.size = len(self.data)
		self.channels = list()
		for channel in self.data:
			self.channels.append(Channel(channel))


class Channel:
	def __init__(self, data: dict = {}):
		self.data=data
		self.channelId = self.data.get("id")
		self.channelType = self.data.get("type")
		self.last_message_id = self.data.get("last_message_id")
		self.flags = self.data.get("flags")
		self.name = self.data.get("name")
		self.icon = self.data.get("icon")
		self.owner_id = self.data.get("owner_id")
		self.channelMembers = list()
		for member in self.data.get("recipients", {}):
			self.channelMembers.append(UserProfile(member))
		self.members_size = len(self.channelMembers)


class ChannelMessages:
	def __init__(self, data: list = []):
		self.data = data
		self.size = len(self.data)
		self.messages = list()
		for message in self.data:
			self.messages.append(Message(message))



"""
Unofficial library for creating user bots for Discord.  
Allows automating actions, managing an account, and interacting with the Discord API without using an official bot.
Author: alx0rr
"""

from .utils.objects import *
from .utils import exceptions, log, get_userId_from_token
from .utils.logger import logging

from .client import Client
#from .async_client import AsyncClient


def set_log_level(level: int):
	"""
	Sets the logging level.

	:param level: The new logging level (e.g., logging.DEBUG, logging.ERROR).
	"""
	log.set_level(level)

__title__ = 'udiscord'
__author__ = 'Xsarz'
__license__ = 'MIT'
__copyright__ = 'Copyright 2025 alx0rr'
__version__ = '1.3.8'


from requests import get
try:__newest__ = get("https://pypi.org/pypi/udiscord/json").json().get("info", {}).get("version", __version__)
except:__newest__=__version__
if __version__ != __newest__:
	log.warning(f'{__title__} made by {__author__}. Please update the library. Your version: {__version__}  A new version: {__newest__}')


from .. import AvatarDecorationData


class LoginInfo:
	def __init__(self, data: dict = {}):
		self.data = data
		user_settings = self.data.get("user_settings", {})
		
		self.token = self.data.get("token")
		self.userId = self.data.get("user_id")
		self.locale = user_settings.get("locale")
		self.theme = user_settings.get("theme")
		self.ticket: str = self.data.get("ticket")
		self.challenge: str = self.data.get("challenge")


class ConnectedAccount: 
	def __init__(self, data: dict):
		self.data: dict = data
		self.visibility: int = data.get("visibility")
		self.verified: bool = data.get("verified")
		self.type: str = data.get("type")
		self.two_way_link: bool = data.get("two_way_link")
		self.show_activity: bool = data.get("show_activity")
		self.revoked: bool = data.get("revoked")
		self.name: bool = data.get("name")
		self.metadata_visibility: int = data.get("metadata_visibility")
		self.id: int = data.get("id")
		self.friend_sync: bool = data.get("friend_sync")



class User:
	def __init__(self, data: dict):
		self.data: dict = data
		self.username: str = data.get("username")
		self.public_flags: int = data.get("public_flags")
		self.primary_guild = data.get("primary_guild")
		self.id: str = data.get("id")
		self.global_name: str = data.get("global_name")
		self.discriminator: str = data.get("discriminator")
		self.clan = data.get("clan")
		self.avatar_decoration_data = AvatarDecorationData(data.get("avatar_decoration_data", {}) or {})
		self.avatar: str = data.get("avatar")



class Profile:
	def __init__(self, data: dict):
		self.data: dict = data
		self.verified: bool = data.get("verified")
		self.username: str = data.get("username")
		self.purchased_flags: int = data.get("purchased_flags")
		self.pronouns: str = data.get("pronouns")
		self.primary_guild = data.get("primary_guild")
		self.premium_type: int = data.get("premium_type")
		self.premium: bool = data.get("premium")
		self.phone: int = data.get("phone")
		self.nsfw_allowed: bool = data.get("nsfw_allowed")
		self.mobile: bool = data.get("mobile")
		self.mfa_enabled: bool = data.get("mfa_enabled")
		self.id: int = data.get("id")
		self.global_name: str = data.get("global_name")
		self.flags: int = data.get("flags")
		self.email: int = data.get("email")
		self.discriminator: str = data.get("discriminator")
		self.desktop: bool = data.get("desktop")
		self.clan = data.get("clan")
		self.bio: str = data.get("bio")
		self.banner_color: str = data.get("banner_color")
		self.banner = data.get("banner")
		self.avatar_decoration_data = data.get("avatar_decoration_data")
		self.avatar: str = data.get("avatar")
		self.accent_color: int = data.get("accent_color")



class Session:
	def __init__(self, data: dict):
		self.data = data
		self.status: str = data.get("status")
		self.session_id: str = data.get("session_id")
		self.activities: list = data.get("activities", [])
		self.id_hash: str = data.get("id_hash")
		self.approx_last_used_time: str = data.get("approx_last_used_time")

		client_info = data.get("client_info", {})
		self.version: int = client_info.get("version")
		self.os: str = client_info.get("os")
		self.client: str = client_info.get("client")


class PrivateChannel:
	def __init__(self, data: dict):
		self.data = data
		self.type: int = data.get("type")
		self.safety_warnings: list = data.get("safety_warnings", [])
		self.recipient_flags: int = data.get("recipient_flags")
		self.last_message_id: str = data.get("last_message_id")
		self.is_spam: bool = data.get("is_spam")
		self.is_message_request_timestamp: bool = data.get("is_message_request_timestamp")
		self.is_message_request: bool = data.get("is_message_request")
		self.id: str = data.get("id")
		self.flags: int = data.get("flags")
		self.recipients: list[User] = list()

		for recipient in data.get("recipients", []):
			self.recipients.append(User(recipient))


class Relationship:
	def __init__(self, data: dict):
		self.data: dict = data
		self.user_ignored: bool = data.get("user_ignored")
		self.type: int = data.get("type")
		self.since: str = data.get("since")
		self.nickname: str = data.get("nickname")
		self.is_spam_request: bool = data.get("is_spam_request")
		self.id: str = data.get("id")
		self.user = User(data.get("user"))



class UserSettings:
	def __init__(self, data: dict):
		self.data: dict = data


class AccountInfo:
	def __init__(self, data: dict):
		self.data: dict = data
		self.resume_gateway_url: str = data.get("resume_gateway_url")
		self.static_client_session_id: str = data.get("static_client_session_id")
		self.tutorial = data.get("tutorial")
		self.session_type: str = data.get("session_type")
		self.country_code: str = data.get("country_code")
		self.session_id: str = data.get("session_id")
		self.v: int = data.get("v")
		self.friend_suggestion_count: int = data.get("friend_suggestion_count")
		self.explicit_content_scan_version: int = data.get("explicit_content_scan_version")
		self.user_settings_proto: str = data.get("user_settings_proto")
		self.auth_session_id_hash: str = data.get("auth_session_id_hash")
		self.api_code_version: int = data.get("api_code_version")
		self.analytics_token: str = data.get("analytics_token")
		self.broadcaster_user_ids: list = data.get("broadcaster_user_ids", [])
		self._trace: list = data.get("_trace")
		self.guild_experiments: list = data.get("guild_experiments", [])
		self.experiments: list = data.get("experiments", [])
		self.connected_accounts: list[ConnectedAccount] = list()
		self.user: Profile = Profile(data.get("user", {}))
		self.sessions: list[Session] = list()
		self.geo_ordered_rtc_regions: list[str] = data.get("geo_ordered_rtc_regions", [])
		self.private_channels: list[PrivateChannel] = list()
		self.relationships: list[Relationship] = list()
		self.game_relationships: list = data.get("game_relationships", []) #idk
		
		
		self.guilds: list[dict] = data.get("guilds", [])
		self.presences: list[dict] = data.get("presences", [])
		self.experiments: list[dict] = data.get("experiments", [])
		self.read_state: list[dict] = data.get("read_state", [])
		self.notification_settings: dict = data.get("notification_settings", )
		self.guild_join_requests: list[dict] = data.get("guild_join_requests", [])
		self.auth: dict = data.get("auth")
		self.user_settings: UserSettings = UserSettings(data.get("user_settings", {}))
		self.notes: dict = data.get("notes")
		self.consents: dict = data.get("consents")
		self.user_guild_settings: list[dict] = data.get("user_guild_settings")


		for account in data.get("connected_accounts", []):
			self.connected_accounts.append(ConnectedAccount(account))
		
		for session in data.get("sessions", []):
			self.sessions.append(Session(session))

		for private_channel in data.get("private_channels", []):
			self.private_channels.append(PrivateChannel(private_channel))

		for relationship in data.get("relationships", []):
			self.relationships.append(Relationship(relationship))


from json import loads
from urllib.parse import quote
from typing import BinaryIO
from random import randint
from os.path import splitext

from .utils.objects import *
from .utils.requester import Requester
from .utils import exceptions, get_wav_audio_metadata, get_ogg_audio_metadata

from .ws import Socket

class Client(Socket):
	"""
	The main client class of the library for Discord user bots.
	Inherits from Socket to work with Discord's WebSocket API in real time.  
	Provides methods for automating actions, managing the account, and interacting with Discord.

	Arguments :
	- proxies: dict = None — proxy for the connection.
	- socket_enable: bool = True — whether to connect the socket (if you disconnect it, events and other related things will not work).
	- sock_trace: bool = False — enables WebSocket connection debugging.
	- detailed_error: bool = False — more detailed error output mode
	- user_agent: str = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/111.0" — user agent string used for requests.
	- os: str = "Windows" — operating system identifier.
	- browser: str = "Firefox" — browser identifier.
	- device: str = "" — device identifier.

	Data available in the class (besides functions):
	- Client.token: str — token after authorization.
	- Client.userId: int — account ID after logging in.
	- Client.account: AccountInfo — a class containing all account information received after connecting to the socket.
	"""


	def __init__(self, proxies: dict = None, socket_enable: bool = True, sock_trace: bool = False, detailed_error: bool = False,
			  user_agent: str = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/111.0", os: str = "Windows", browser: str = "Firefox", device: str = ""):
		self.socket_enable=socket_enable
		self.req = Requester(user_agent)
		self.proxies = proxies
		self.account: AccountInfo = AccountInfo({})
		Socket.__init__(self, os, browser, device, sock_trace, detailed_error)

		self.add_handler(EventType.READY, self.__on_connect)
	

	def __repr__(self) -> str:
		attrs = [
			('token', self.token),
			('userId', self.userId)
		]
		inner = ' '.join('%s=%r' % t for t in attrs)
		return f"<class '{self.__class__.__name__}' {inner}>"


	def __del__(self):
		if self.token:
			try:self.logout()
			except: pass



	def __on_connect(self, event: AccountInfo):
		self.account = event

	@property
	def token(self):
		return self.req.token

	@property
	def userId(self):
		return self.req.userId

	@property
	def user_agent(self):
		return self.req.user_agent



	def login(self, login: str, password: str, login_source: str = None, gift_code_sku_id: str = None) -> LoginInfo:
		"""
		`Log in to your account`
		"""

		data = {
			"login": login,
			"password": password,
			"undelete": False,
			"captcha_key": None,
			"login_source": login_source,
			"gift_code_sku_id": gift_code_sku_id
		}

		resp = LoginInfo(self.req.make_request(method="POST", endpoint="/auth/login", body=data, proxies=self.proxies).json())
		self.req.token = resp.token
		self.req.userId = resp.userId
		if self.socket_enable:self.connect()
		return resp


	def login_token(self, token: str) -> LoginInfo:
		"""
		`Log in to your account by token`
		"""
		self.req.token = token
		info = self.conditional_start()
		if not info.token:
			self.req.token = None
			raise exceptions.InvalidAuthorizationToken(info.data)
		self.req.userId = info.userId
		if self.socket_enable:self.connect()
		return info

	def qrcode_auth(self, code: str, temporary_token: bool = False) -> int:
		"""
		Authenticates the user via QR code.

		This method performs remote authentication using a QR code scanned from 
		the Discord app (e.g., https://discord.com/ra/{code}).

		:param code: The unique authentication code from the QR link.
		:param temporary_token: If True, a temporary token will be issued instead of a permanent session.
		:return: The HTTP status code of the authentication request.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		handshake = self.create_remote_auth_handshake(code)
		return self.remote_auth(handshake, temporary_token)


	def create_remote_auth_handshake(self, code: str) -> str:
		"""
		Initiates a remote authentication handshake.

		Sends a request to begin the authentication process using the QR code.

		:param code: The authentication code extracted from the QR link.
		:return: A handshake token required to complete authentication.
		"""
		data = {
			"fingerprint": code
		}
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="POST", endpoint="/users/@me/remote-auth", body=data, proxies=self.proxies).json().get("handshake_token")


	def remote_auth(self, handshake: str, temporary_token: bool = False) -> int:
		"""
		Completes remote authentication using the handshake token.

		:param handshake: The handshake token obtained from `create_remote_auth_handshake`.
		:param temporary_token: If True, a temporary token will be issued.
		:return: The HTTP status code (204 on success).
		"""
		data = {
			"handshake_token": handshake,
			"temporary_token": temporary_token
		}
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="POST", endpoint="/users/@me/remote-auth/finish", body=data, proxies=self.proxies, allowed_code=204).status_code



	def logout(self) -> int:
		"""
		`Sign out of your account`

		:return: HTTP status code of the request. (204 on success)
		"""

		data = {
			"provider": None,
			"voip_provider": None
		}
		if not self.token: raise exceptions.UnauthorizedClientError
		resp = self.req.make_request(method="POST", endpoint="/auth/logout", body=data, allowed_code=204, proxies=self.proxies).status_code
		self.req.token = None
		self.req.userId = None
		if self.active:self.disconnect()
		self.account = AccountInfo({})
		return resp


	def get_channel_messages(self, channelId: int, size: int = 50) -> ChannelMessages:
		"""
		`Get a list of messages in a channel`
			**channelId - ID of the selected channel [TYPE: int]

			*size - Message list length [TYPE: int] [Default: 50]
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return ChannelMessages(self.req.make_request(method="GET", endpoint=f"/channels/{channelId}/messages?limit={size}", proxies=self.proxies).json())

	def get_user_info(self, userId: int, with_mutual_guilds: bool = True, with_mutual_friends_count: bool = True) -> UserProfile:
		"""
		`Get user information`
			**userId - ID of the selected user [TYPE: int]

			*with_mutual_guilds - Get shared servers [TYPE: bool] [Default: True]
			*with_mutual_friends_count - Get mutual friends [TYPE: bool] [Default: True]
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return UserProfile(self.req.make_request(method="GET", endpoint=f"/users/{userId}/profile?with_mutual_guilds={with_mutual_guilds}&with_mutual_friends_count={with_mutual_friends_count}", proxies=self.proxies).json())


	def get_me_info(self) -> dict:
		"""
		Retrieves information about the current user.

		:return: A dictionary containing the user's profile data.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return Profile(self.req.make_request(method="GET", endpoint="/users/@me", proxies=self.proxies).json())


	def online_status(self, mode: int = 0) -> str:
		"""
		Sets the user's online status.

		Available statuses:
		- 0: Online
		- 1: Idle
		- 2: Do Not Disturb
		- 3: Invisible

		:param mode: Numeric status code (0-3).
		:return: The Base64-encoded settings value.
		:raises exceptions.WrongModeError: If an invalid status code is provided.
		"""
		modes = {
			0: "WgoKCAoGb25saW5l",  # online
			1: "WggKBgoEaWRsZQ==",  # idle
			2: "WgcKBQoDZG5k",  # do not disturb
			3: "Wg0KCwoJaW52aXNpYmxl"  # invisible
		}

		if mode not in modes:
			raise exceptions.WrongModeError(mode)
		if not self.token: raise exceptions.UnauthorizedClientError

		data = {"settings": modes.get(mode)}
		return self.req.make_request(method="PATCH", endpoint="/users/@me/settings-proto/1", body=data, proxies=self.proxies).json()["settings"]

	def get_my_channels(self) -> Channels:
		"""
		`Get a list of channels on an account`
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return Channels(self.req.make_request(method="GET", endpoint=f"/users/@me/channels", proxies=self.proxies).json())

	def get_my_guilds(self) -> Guilds:
		"""
		`Get the servers you belong to`
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return Guilds(self.req.make_request(method="GET", endpoint=f"/users/@me/guilds", proxies=self.proxies).json())

	def send_message(self, channelId: int, message: str) -> Message:
		"""
		`Send a message to a channel`
			**channelId - Channel ID [TYPE: int]
			**message - Your message [TYPE: str]
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		data = {"content": message}
		return Message(self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/messages", body=data, proxies=self.proxies).json())

	def send_file(self, channelId: int, files: list[BinaryIO]) -> Message:

		"""
		`Send a file to a channel (file, video, image, ect.)`
		Args:
			channelId - Channel ID [TYPE: int]
			files - List of files ( with open("file", "rb") as file:) [TYPE: list[BinaryIO]]
		"""

		if not self.token: raise exceptions.UnauthorizedClientError

		attachments = self.upload_chat_media(channelId, files)
		data = {
		"content":"",
		"type": 0,
		"channel_id": channelId,
		"attachments": attachments
		}
		return Message(self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/messages", body=data, proxies=self.proxies).json())
	
	def send_voice_message(self, channelId: int, file: BinaryIO) -> Message:
		"""
		`Sending a voice message to a chat`

		`Args:`
			channelId - Channel ID [TYPE: int]
			file - audio file( with open("file", "rb") as file:) [TYPE: list[BinaryIO]]

		`Supported audio files:`
			.wav
			.ogg
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		filename = getattr(file, 'name', None)
		extension = splitext(filename)[1] if filename else None
		
		match (extension):
			case '.wav':
				duration_secs, waveform = get_wav_audio_metadata(file)
			case '.ogg':
				duration_secs, waveform = get_ogg_audio_metadata(file)
			case _:
				raise exceptions.WrongAudioType(f"Unsupported file type: {extension}")				

		attachments = self.upload_chat_media(channelId, [file])
		attachments[0]["duration_secs"] = duration_secs
		attachments[0]["waveform"] = waveform

		data = {
		"content":"",
		"type": 0,
		"channel_id": channelId,
		"attachments": attachments
		}
		return Message(self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/messages", body=data, proxies=self.proxies).json())



	def edit_message(self, channelId: int, message: str, messageId: int) -> Message:
		"""
		`Edit message`
			**channelId - Channel ID [TYPE: int]
			**message - Your message [TYPE: str]
			**messageId - Message ID [TYPE: int]
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		data = {"content": message}
		return Message(self.req.make_request(method="PATCH", endpoint=f"/channels/{channelId}/messages/{messageId}", body=data, proxies=self.proxies).json())

	def reply_message(self, channelId: int, message: str, messageId: int, guildId: int = None) -> Message:
		"""
		`Reply to message`
			**channelId - Channel ID [TYPE: int]
			**message - Your message [TYPE: str]
			**messageId - Message ID [TYPE: int]
			**guildId - server ID [TYPE: int | None]
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		data = {"content": message, "message_reference": {
			"channel_id": channelId,
			"message_id": messageId
		}}
		if guildId: data["guild_id"]=guildId
		return Message(self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/messages", body=data, proxies=self.proxies).json())



	def delete_message(self, channelId: int, messageId: int) -> int:
		"""
		`Delete a message from a channel`
			**channelId - Channel ID [TYPE: int]
			**messageId - Message ID [TYPE: int]
			
			:return: HTTP status code of the request. (204 on success)
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="DELETE", endpoint=f"/channels/{channelId}/messages/{messageId}", allowed_code=204, proxies=self.proxies).status_code


	def mark_message_as_unread(self, channelId: int, messageId: int) -> int:
		"""
		Marks a specific message as unread in a channel.

		Args:
			channelId (int): The ID of the channel containing the message.
			messageId (int): The ID of the message to be marked as unread.

		Returns:
			int: The HTTP status code of the response.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/messages/{messageId}/ack", body={"manual":True,"mention_count":0}, proxies=self.proxies).status_code


	def typing(self, channelId: int) -> int:
		"""
		Sends a typing indicator in a channel.

		Args:
			channelId (int): The ID of the channel where the typing indicator should be sent.

		Returns:
			int: The HTTP status code of the response.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/typing", proxies=self.proxies, allowed_code=204).status_code


	def get_invite_info(self, invite_code: str, with_counts: bool = True, with_expiration: bool = True, with_permissions: bool = False) -> Invite:
		"""
		`Retrieve information about an invite`
			**invite_code - Invite code [TYPE: str]
			**with_counts - Include usage counts (default: True) [TYPE: bool]
			**with_expiration - Include expiration details (default: True) [TYPE: bool]
			**with_permissions - Include permissions information (default: False) [TYPE: bool]
		"""
		return Invite(self.req.make_request(method="GET", endpoint=f"/invites/{invite_code}?with_counts={with_counts}&with_expiration={with_expiration}&with_permissions={with_permissions}", proxies=self.proxies).json())
	
	def get_invites(self, channelId: int) -> Invites:
		"""
		`Retrieve all invites for a channel`
			**channelId - Channel ID [TYPE: int]
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return Invites(self.req.make_request(method="GET", endpoint=f"/channels/{channelId}/invites", proxies=self.proxies).json())


	def create_invite(self, channelId: int, max_age: int = 86400) -> Invite:
		"""
		Creates an invite for the specified channel.

		:param channelId: The ID of the channel for which the invite is created.
		:param max_age: The invite's lifetime in seconds (default is 86400 seconds = 24 hours).
		:return: An Invite object.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		Invite(self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/invites", proxies=self.proxies, body={"max_age": max_age}).json())


	def delete_invite(self, invite_code: str) -> int:
		"""
		Deletes an invite from the server.

		Args:
			invite_code (str): The unique code of the invite to be deleted.

		Returns:
			int: The HTTP status code of the response.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="DELETE", endpoint=f"/invites/{invite_code}", allowed_code=204, proxies=self.proxies).status_code


	def get_audit_logs(self, guildId: int, userId: int = None, action_type: int = AuditAction.AllActions, limit: int = 50) -> dict:
		"""
		Retrieves audit logs for a specific guild.

		Args:
			guildId (int): The ID of the guild.
			userId (int, optional): The ID of a specific user to filter logs by. Defaults to None.
			action_type (int, AuditAction.Some, optional): The type of action to filter logs by. Defaults to None.
			limit (int, optional): The maximum number of logs to retrieve. Defaults to 50.

		Returns:
			AuditLog: A class containing the audit log data.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return AuditLog(self.req.make_request(method="GET", endpoint=f"/guilds/{guildId}/audit-logs?limit={limit}{f'&user_id={userId}' if userId else ''}{f'&action_type={action_type}' if action_type else ''}", proxies=self.proxies).json())


	def leave_guild(self, guildId: int, lurking: bool = False) -> int:
		"""
		Leaves a guild.

		:param guildId: The ID of the guild to leave.
		:param lurking: If True, the user will remain in the guild in a "lurking" (invisible presence) mode.
		:return: HTTP status code of the request. (204 on success)
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="DELETE", endpoint=f"/users/@me/guilds/{guildId}", allowed_code=204, proxies=self.proxies, body={"lurking": lurking}).status_code


	def conditional_start(self) -> LoginInfo:
		"""
		`Get information about the current session`
		"""
		return LoginInfo(self.req.make_request(method="POST", endpoint=f"/auth/conditional/start", proxies=self.proxies).json())

	def get_location_metadata(self) -> LocationMetadata:
		"""
		`Retrieve metadata about the current location`
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return LocationMetadata(self.req.make_request(method="GET", endpoint=f"/auth/location-metadata", proxies=self.proxies).json())

	def check_username_taken(self, username	: str) -> bool:
		"""
		`Checks if the username is busy`

		:param username: username for check
		:return: True or False
		"""
		return self.req.make_request(method="POST", endpoint="/users/@me/pomelo-attempt" if self.token else "/unique-username/username-attempt-unauthed", proxies=self.proxies, body={"username": username}).json()["taken"]


	def resend_verify_code(self) -> int:
		"""
		`resend the confirmation code by email`

		:return: HTTP status code of the request. (204 on success)
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="POST", endpoint=f"/auth/verify/resend", proxies=self.proxies, allowed_code=204).status_code
	
	def forgot_password_request(self, email: str) -> int:
		"""
		Sends a password reset email.

		:param email: The email address associated with the account.
		:return: HTTP status code of the request. (204 on success)
		"""
		return self.req.make_request(method="POST", endpoint=f"/auth/forgot", body={"login": email}, proxies=self.proxies, allowed_code=204).status_code
	
	def edit_profile(self, bio: str = None, pronouns: str = None, accent_color: str = None) -> UserProfile:
		"""
		Edits the user's profile.

		:param bio: New biography text (optional).
		:param pronouns: New pronouns (optional) (like "he / his").
		:param accent_color: New accent color in hex format (e.g., "#FF5733") (optional).
		:return: UserProfile object.
		:raises: exceptions.ArgumentNotSpecifiedError if no parameters are provided.
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		data = {}
		if bio is None and pronouns is None and accent_color is None: raise exceptions.ArgumentNotSpecifiedError
		if bio: data["bio"]=bio
		if pronouns: data["pronouns"]=pronouns
		if accent_color: data["accent_color"]=int(accent_color[1:], 16)
		return UserProfile(self.req.make_request(method="PATCH", endpoint=f"/users/@me/profile", proxies=self.proxies, body=data).json())


	def delete_friend(self, userId: int) -> int:
		"""
		Removes a user from the friend list.

		:param userId: The ID of the user to remove.
		:return: HTTP status code of the request. (204 on success)
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="DELETE", endpoint=f"/users/@me/relationships/{userId}", proxies=self.proxies, allowed_code=204).status_code

	def accept_friend_request(self, userId: int) -> int:
		"""
		Accepts a friend request from a user.

		:param userId: The ID of the user whose request is being accepted.
		:return: HTTP status code of the request. (204 on success)
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="PUT", endpoint=f"/users/@me/relationships/{userId}", proxies=self.proxies, body={}, allowed_code=204).status_code
	
	def ignore_friend_request(self, userId: int) -> int:
		"""
		Ignores (declines) a friend request from a user.

		:param userId: The ID of the user whose request is being ignored.
		:return: HTTP status code of the request. (204 on success)
		""" 
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.delete_friend(userId)


	def get_my_sessions(self) -> list[Session]:
		"""
		`Get a list of sessions (devices on which you were logged in)`
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		return [Session(session) for session in self.req.make_request(method="GET", endpoint="/auth/sessions", proxies=self.proxies).json().get("user_sessions", [])]



	def logout_sessions(self, sessionIds_hashes: list | str, password: str | None = None) -> str:
		"""
		`Log out of your account on other devices`
		:param sessionIds_hashes: hashed ID or list that you can get from the `client.get_my_sessions` function
		:param password: password for the account (will be used in case of an authentication request)
		"""
		if not self.token: raise exceptions.UnauthorizedClientError
		if isinstance(sessionIds_hashes, str): sessionIds_hashes = [sessionIds_hashes]
		data = {
			"session_id_hashes": sessionIds_hashes
		}
		try:
			return self.req.make_request(method="POST", endpoint=f"/auth/sessions/logout", proxies=self.proxies, body=data).json()
		except exceptions.TwoFactorAuthenticationRequired as e:
			mfa_data = loads(str(e)).get("mfa")
			ticket = mfa_data.get("ticket")
			methods = mfa_data.get("methods", [])

			for method in methods:
				if method.get("type") == "password" and password is not None:
					return self._mfa_finish(ticket=ticket, method="password", data=password)
			
			raise e


	def _mfa_finish(self, mfa_type: str, ticket: str, data) -> str:
		"""
		`Helper function for passing multi-factor authentication.`
		"""
		
		_data = {
			"mfa_type": mfa_type,
			"ticket": ticket,
			"data": data
		}
		return self.req.make_request(method="POST", endpoint=f"/mfa/finish", proxies=self.proxies, body=_data).json().get("token")
	


	def kick_guild_member(self, guildId: int, userId: str, reason: str = None) -> int:
		"""
		`Kick a member from the guild`

		:param guildId: ID of the guild
		:param userId: ID of the user to be kicked
		:param reason: Reason for kicking the user (optional)
		:return: HTTP status code (204 if successful)
		"""
		
		if not self.token: raise exceptions.UnauthorizedClientError
		return self.req.make_request(method="DELETE", endpoint=f"/guilds/{guildId}/members/{userId}", proxies=self.proxies, allowed_code=204, headers={
			"x-audit-log-reason": quote(reason) if reason else ''}
		).status_code
	

	def ban_guild_member(self, guildId: int, userId: str, reason: str = None, delete_user_messages: int = 0) -> int:

		"""
		`Ban a member from the guild`

		:param guildId: ID of the guild
		:param userId: ID of the user to be banned
		:param reason: Reason for banning the user (optional)
		:param delete_user_messages: Number of seconds of messages to delete (default: 0)
		:return: HTTP status code (204 if successful)
		"""

		if not self.token: raise exceptions.UnauthorizedClientError
		data = {"delete_message_seconds":delete_user_messages}
		return self.req.make_request(method="PUT", endpoint=f"/guilds/{guildId}/bans/{userId}", body=data, proxies=self.proxies, allowed_code=204, headers={
			"x-audit-log-reason": quote(reason) if reason else ''}
		).status_code


	def unban_guild_member(self, guildId: int, userId: str) -> int:
		"""
		`Unban a member from the guild`
		
		:param guildId: ID of the guild
		:param userId: ID of the user to be unbanned
		:return: HTTP status code (204 if successful)
		"""

		if not self.token: raise exceptions.UnauthorizedClientError
		raise NotImplementedError
		#TODO
		return self.req.make_request(method="DELETE", endpoint=f"/guilds/{guildId}/bans/{userId}", proxies=self.proxies, allowed_code=204).status_code

	def get_guild_bans(self, guildId: int, limit: int = 1000) -> list[BannedUser]:
		"""
		`Retrieve a list of banned members in a guild`

		:param guildId: ID of the guild
		:param limit: Maximum number of bans to retrieve (default: 1000)
		:return: List of BannedUser objects
		"""

		if not self.token: raise exceptions.UnauthorizedClientError
		result = self.req.make_request(method="GET", endpoint=f"/guilds/{guildId}/bans?limit={limit}", proxies=self.proxies).json()
		return [BannedUser(data) for data in result]
	

	def get_avatar_image(self, userId: int, avatar: str, size: int = 96) -> bytes:
		"""
		`Retrieve a user's avatar image`

		:param userId: ID of the user
		:param avatar: Avatar hash
		:param size: Size of the avatar image (default: 96px)
		:return: Image bytes
		"""
		return self.req.make_request(method="GET", endpoint=f"/avatars/{userId}/{avatar}.webp?size={size}", proxies=self.proxies, api="https://cdn.discordapp.com").content


	def get_guild_icon(self, guildId: int, icon: str, size: int = 240) -> bytes:

		"""
		`Retrieve a guild avatar image`

		:param guildId: ID of the guild
		:param icon: guild icon hash
		:param size: Size of the icon image (default: 240)
		:return: Image bytes
		"""

		return self.req.make_request(method="POST", endpoint=f"/icons/{guildId}/{icon}.webp?size={size}", proxies=self.proxies, api="https://cdn.discordapp.com").content
	

	def upload_chat_media(self, channelId: int, files: list[BinaryIO]) -> dict:
		"""
		`Uploading files to the discord server. auxiliary function`
		"""
		f = list()
		files_data = list()
		for file in files:
			fd = file.read()
			files_data.append(fd)
			f.append(
				{"filename":file.name,"file_size":len(fd),"id":str(randint(60, 70))}
			)
		
		result: list[dict] = self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/attachments", proxies=self.proxies, body={"files":f}).json().get("attachments", [])

		attachments = list()

		for num, attachment in enumerate(result):
			self.req.make_request(method="PUT", proxies=self.proxies, api=attachment.get("upload_url"), body=files_data[num])
			attachments.append({
				"filename": f[num].get("filename"),
				"id": f[num].get("id"),
				"uploaded_filename": attachment.get("upload_filename")

			})
		return attachments
	




class LocationMetadata:
    def __init__(self, data: dict):
        self.data: dict = data
        self.consent_required: bool = data.get("consent_required")
        self.country_code: str = data.get("country_code")
        self.promotional_email_opt_in: PromotionalEmailOptIn = PromotionalEmailOptIn(data.get("promotional_email_opt_in", {}))


class PromotionalEmailOptIn:
    def __init__(self, data: dict):
        self.data: dict = data
        self.required: bool = data.get("required")
        self.pre_checked: bool = data.get("pre_checked")


from websocket import WebSocketApp, enableTrace
from threading import Thread
from time import sleep
from ujson import dumps, loads

from ..utils.requester import Requester
from .. import log
from .socket_handler import Handler
from traceback import format_exc

class Socket(Handler):
	heartbeat_started: bool = False
	req: Requester
	active: bool = False
	heartbeat_interval: int = None

	def __init__(self, os: str, browser: str, device: str, sock_trace: bool = False, detailed_error: bool = False):
		self.socket_url = "wss://gateway.discord.gg"
		self.socket = None
		self.detailed_error = detailed_error
		enableTrace(sock_trace)
		self.os, self.browser, self.device = os, browser, device
		Handler.__init__(self)

	def connect(self):
		try:
			if self.active: return
			log.debug(f"[socket][start] Starting Socket")

			self.socket = WebSocketApp(
				f"{self.socket_url}/?encoding=json&v=9",
				header = self.req.headers(),
				on_message=self.resolve
			)
			self.socket_thread = Thread(target=self.socket.run_forever)
			self.socket_thread.start()
			Thread(target=self.start_heartbeat).start()
			self.active=True
			log.debug(f"[socket][start] Socket Started")
		except Exception as e:
			log.error(f"[socket][start] Error while starting Socket : {e}{'' if not self.detailed_error else f'\n{format_exc()}'}")


	def disconnect(self):
		if not self.active:return
		log.debug(f"[socket][close] Closing Socket")
		try:
			self.socket.close()
			self.socket_thread = None
			self.active = False
			log.debug(f"[socket][close] Socket closed")
		except Exception as e:
			log.error(f"[socket][close] Error while closing Socket : {e}{'' if not self.detailed_error else f'\n{format_exc()}'}")


	def resolve(self, ws, data):
		try:
			data = loads(data)
			if data.get("op") == 10:
				self.heartbeat_interval = data['d']['heartbeat_interval'] / 1000
				return
			self.handle_data(data)
		except Exception as e:
			log.warning(f"[socket][resolve] Error while resolve data : {e}{'' if not self.detailed_error else f'\n{format_exc()}'}")


	def start_heartbeat(self):
		sleep(0.5)
		identify_payload = {
			"op": 2,
			"d": {
				"token": self.req.token,
				"properties": {
					"os": self.os,
					"browser": self.browser,
					"device": self.device
				}
			}
		}
		for i in range(5):
			try:
				self.send(identify_payload)
				break
			except Exception as e:
				log.error("[socket][start] Failed connect to discord socket. reconnecting...")
				sleep(1.5)
		else:
			log.critical("[socket][start] Failed connect to discord socket.")

		while True:
			if not self.heartbeat_interval: continue
			sleep(self.heartbeat_interval)
			if not self.active:return
			self.send({"op": 1, "d": None})

	def send(self, data: str | dict | bytes):
		if not self.active or not self.socket:
			log.debug(f"[socket][send][error]: socket is disabled or not running.")
			return
		log.debug(f"[socket][send]: {data}")
		data = data if isinstance(data, bytes) else dumps(data)
		self.socket.send(data)


from . import Message


class LoginInfo:
	def __init__(self, data: dict = {}):
		self.data = data
		user_settings = self.data.get("user_settings", {})
		
		self.token = self.data.get("token")
		self.userId = self.data.get("user_id")
		self.locale = user_settings.get("locale")
		self.theme = user_settings.get("theme")
		self.ticket: str = self.data.get("ticket")
		self.challenge: str = self.data.get("challenge")




class ChannelMessages:
	def __init__(self, data: list = []):
		self.data = data
		self.size = len(self.data)
		self.messages = list()
		for message in self.data:
			self.messages.append(Message(message))



class UserProfile:
	def __init__(self, data: dict = {}):
		self.data = data
		user = self.data.get("user", {})

		self.userId = user.get("id")
		self.username = user.get("username")
		self.global_name = user.get("global_name")
		self.display_name = user.get("display_name")
		self.avatar = user.get("avatar")
		self.avatar_decoration = user.get("avatar_decoration")
		self.discriminator = user.get("discriminator")
		self.public_flags = user.get("public_flags")
		self.flags = user.get("flags")
		self.banner = user.get("banner")
		self.banner_color = user.get("banner_color")
		self.accent_color = user.get("accent_color")
		self.bio = user.get("bio")
		self.premium_since = self.data.get("premium_since")
		self.premium_type = self.data.get("premium_type")
		self.premium_guild_since = self.data.get("premium_guild_since")
		self.profile_themes_experiment_bucket = self.data.get("profile_themes_experiment_bucket")
		self.mutual_friends_count = self.data.get("mutual_friends_count")
		self.mutual_guilds = self.data.get("mutual_guilds")
		self.connected_accounts = self.data.get("connected_accounts")


class Channels:
	def __init__(self, data: list = []):
		self.data = data
		self.size = len(self.data)
		self.channels = list()
		for channel in self.data:
			self.channels.append(Channel(channel))


class Channel:
	def __init__(self, data: dict = {}):
		self.data=data
		self.channelId = self.data.get("id")
		self.channelType = self.data.get("type")
		self.last_message_id = self.data.get("last_message_id")
		self.flags = self.data.get("flags")
		self.name = self.data.get("name")
		self.icon = self.data.get("icon")
		self.owner_id = self.data.get("owner_id")
		self.channelMembers = list()
		for member in self.data.get("recipients", {}):
			self.channelMembers.append(UserProfile(member))
		self.members_size = len(self.channelMembers)

class Guilds:
	def __init__(self, data: list = []):
		self.data = data
		self.size = len(self.data)
		self.guilds: list[Guild] = list()
		for guild in self.data:
			self.guilds.append(Guild(guild))


class Guild:
	def __init__(self, data: dict = {}):
		self.data = data
		self.guildId = self.data.get("id")
		self.guildName = self.data.get("name")
		self.icon = self.data.get("icon")
		self.owner = self.data.get("owner")
		self.permissions = self.data.get("permissions")
		self.features = self.data.get("features", [])



class BannedUser:
	def __init__(self, data: dict):
		self.data = data

		user: dict = self.data.get("user", {})

		self.userId = user.get("id")
		self.username = user.get("username")
		self.avatar = user.get("avatar")
		self.global_name = user.get("global_name")
		self.discriminator = user.get("discriminator")
		self.public_flags = user.get("public_flags")
		self.flags = user.get("flags")
		self.banner = user.get("banner")
		self.accent_color = user.get("accent_color")
		self.banner_color = user.get("banner_color")
		self.collectibles = user.get("collectibles")
		self.avatar_decoration_data = user.get("avatar_decoration_data")
		self.primary_guild = user.get("primary_guild")
		self.clan = user.get("clan")
		self.reason = data.get("reason")
	

class AuditLog:
	def __init__(self, data: dict):
		self.data = data
		self.audit_log_entries = data.get("audit_log_entries", [])
		self.users = data.get("users", [])
		self.integrations = data.get("integrations", [])
		self.webhooks = data.get("webhooks", [])
		self.guild_scheduled_events = data.get("guild_scheduled_events", [])
		self.threads = data.get("threads", [])
		self.application_commands = data.get("application_commands", [])
		self.auto_moderation_rules = data.get("auto_moderation_rules", [])


from .message import Message

from .audit import *
from .location_metadata import *
from .user import *
from .guild import *
from .channel import *
from .invites import *

from .event.account_info import AccountInfo, Profile, Session, LoginInfo
from .event.event import Event

from .args import *


from json import loads, JSONDecodeError

class DiscordError(Exception):
	"""
	Base class for all Discord-related errors.
	"""
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)


class LibraryError(Exception):
	"""
	Base class for all library-related errors.
	"""
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)
#++++++++++++++++++++++++++++++++++++++++++++++++++++=



class WrongModeError(LibraryError):
	"""
	An incorrect or unexpected mode is set.
	"""


class WrongAudioType(LibraryError):
	"""
	An incorrect or unexpected mode is set.
	"""


class ArgumentNotSpecifiedError(LibraryError):
	"""
	A required argument was not provided.
	"""


class UnauthorizedClientError(LibraryError):
	"""
	The client is not authorized to perform this action.
	"""


class UnknownError(LibraryError):
	"""
	An unknown error occurred.
	"""

#=====================================================


class RateLimited(DiscordError):
	"""
	Too many requests have been sent in a short period.
	"""


class InsufficientRights(DiscordError):
	"""
	The user does not have the necessary permissions to perform this action.
	"""


class InvalidLoginOrPassword(DiscordError):
	"""
	The provided login or password is incorrect.
	"""


class InvalidAuthorizationToken(DiscordError):
	"""
	The provided authorization token is invalid or expired.
	"""


class IncorrectJsonData(DiscordError):
	"""
	The provided JSON data is malformed or contains invalid values.
	"""


class UnknownInvitation(DiscordError):
	"""
	The specified invitation code is invalid or does not exist.
	"""


class IncorrectServer(DiscordError):
	"""
	The specified server does not exist or cannot be accessed.
	"""


class USERNAME_BLOCKED_UNVERIFIED(DiscordError):
	"""
	The username is blocked due to the account being unverified.
	"""

class CaptchaError(DiscordError):
	"""
	The server returned the captcha to confirm that you are not a robot
	"""


class TwoFactorAuthenticationRequired(DiscordError):
	"""
	If two-factor authentication is required to perform an action (use `client._mfa_finish`)
	"""

errors = {
	20028: RateLimited,
	40062: RateLimited,
	50013: InsufficientRights,
	50035: InvalidLoginOrPassword,
	50109: IncorrectJsonData,
	10006: UnknownInvitation,
	50035: USERNAME_BLOCKED_UNVERIFIED,
	50055: IncorrectServer,
	60003: TwoFactorAuthenticationRequired,
}

def checkException(data):
	try:
		data = loads(data)
		code = data.get("code")
	except JSONDecodeError:
		raise UnknownError(data)
	if not code and any("captcha" in key.lower() for key in data.keys()):raise CaptchaError(data)
	if code in errors: raise errors[code](data)
	else:raise UnknownError(data)


from .logger import Logger
from base64 import b64decode, b64encode
from typing import BinaryIO
from wave import open as wopen
import struct
import base64
from random import randint

log = Logger()


def get_userId_from_token(token: str) -> int:
    bot_id_encoded = token.split(".")[0]
    missing_padding = len(bot_id_encoded) % 4
    if missing_padding:
        bot_id_encoded += "=" * (4 - missing_padding)
    return int(b64decode(bot_id_encoded).decode("utf-8"))


def generate_random_wave(length: int, min_value=-32768, max_value=32767):
    waveform = bytearray()
    
    for _ in range(length):
        # Генерируем случайное значение в диапазоне от min_value до max_value
        sample = randint(min_value, max_value)
        
        # Добавляем значение в байтовый массив в формате 16-битного целого числа
        waveform.extend(struct.pack('<h', sample))  # <h — маленький порядок байтов для 16-битного целого
        
    return waveform


def get_wav_audio_metadata(file: BinaryIO):
    file.seek(0)
    
    with wopen(file, 'rb') as audio:
        duration_secs = audio.getnframes() / audio.getframerate()
        
        frames = audio.readframes(audio.getnframes())
        sampled_frames = frames[:60]
        
        waveform = b64encode(sampled_frames).decode('utf-8')
    
    return duration_secs, waveform


def get_ogg_audio_metadata(file: BinaryIO):
    #TODO
    file.seek(0)

    duration_secs = randint(1, 60)
    waveform = generate_random_wave(max(duration_secs, 10))
    
    waveform_b64 = base64.b64encode(waveform[:60]).decode('utf-8')

    return duration_secs, waveform_b64






class Guilds:
	def __init__(self, data: list = []):
		self.data = data
		self.size = len(self.data)
		self.guilds: list[Guild] = list()
		for guild in self.data:
			self.guilds.append(Guild(guild))


class Guild:
	def __init__(self, data: dict = {}):
		self.data = data
		self.guildId = self.data.get("id")
		self.guildName = self.data.get("name")
		self.icon = self.data.get("icon")
		self.owner = self.data.get("owner")
		self.permissions = self.data.get("permissions")
		self.features = self.data.get("features", [])






class UserProfile:
	def __init__(self, data: dict = {}):
		self.data = data
		user = self.data.get("user", {})

		self.userId = user.get("id")
		self.username = user.get("username")
		self.global_name = user.get("global_name")
		self.display_name = user.get("display_name")
		self.avatar = user.get("avatar")
		self.avatar_decoration = user.get("avatar_decoration")
		self.discriminator = user.get("discriminator")
		self.public_flags = user.get("public_flags")
		self.flags = user.get("flags")
		self.banner = user.get("banner")
		self.banner_color = user.get("banner_color")
		self.accent_color = user.get("accent_color")
		self.bio = user.get("bio")
		self.premium_since = self.data.get("premium_since")
		self.premium_type = self.data.get("premium_type")
		self.premium_guild_since = self.data.get("premium_guild_since")
		self.profile_themes_experiment_bucket = self.data.get("profile_themes_experiment_bucket")
		self.mutual_friends_count = self.data.get("mutual_friends_count")
		self.mutual_guilds = self.data.get("mutual_guilds")
		self.connected_accounts = self.data.get("connected_accounts")



class BannedUser:
	def __init__(self, data: dict):
		self.data = data

		user: dict = self.data.get("user", {})

		self.userId = user.get("id")
		self.username = user.get("username")
		self.avatar = user.get("avatar")
		self.global_name = user.get("global_name")
		self.discriminator = user.get("discriminator")
		self.public_flags = user.get("public_flags")
		self.flags = user.get("flags")
		self.banner = user.get("banner")
		self.accent_color = user.get("accent_color")
		self.banner_color = user.get("banner_color")
		self.collectibles = user.get("collectibles")
		self.avatar_decoration_data = user.get("avatar_decoration_data")
		self.primary_guild = user.get("primary_guild")
		self.clan = user.get("clan")
		self.reason = data.get("reason")


class AvatarDecorationData:
	def __init__(self, data: dict):
		self.data: dict = data
		self.sku_id: str = data.get("sku_id")
		self.expires_at: int = data.get("expires_at")
		self.asset: str = data.get("asset")



class Message:
	def __init__(self, data: dict = {}):
		self.data = data
		self.messageId = self.data.get("id")
		self.messageType = self.data.get("type")
		self.content = self.data.get("content")
		self.channelId = self.data.get("channel_id")
		self.attachments = self.data.get("attachments", [])
		self.embeds = self.data.get("embeds", [])
		self.mentions = self.data.get("mentions", [])
		self.mention_roles = self.data.get("mention_roles", [])
		self.pinned = self.data.get("pinned")
		self.mention_everyone = self.data.get("mention_everyone")
		self.tts = self.data.get("tts")
		self.timestamp = self.data.get("timestamp")
		self.edited_timestamp = self.data.get("edited_timestamp")
		self.flags = self.data.get("flags")
		self.components = self.data.get("components", [])
		self.author = self.Author(self.data.get("author", {}))
		self.referenced_message = self.data.get("referenced_messages")
		self.guildId = self.data.get("guild_id")

	class Author:
		def __init__(self, data: dict = {}):
			self.data = data
			self.userId = self.data.get("id")
			self.username = self.data.get("username")
			self.global_name = self.data.get("global_name")
			self.display_name = self.data.get("display_name")
			self.avatar = self.data.get("avatar")
			self.avatar_decoration = self.data.get("avatar_decoration")
			self.discriminator = self.data.get("discriminator")
			self.public_flags = self.data.get("public_flags")


from .. import log
from ..utils.objects import AccountInfo, Event, Message, EventType
from traceback import format_exc

class Handler:
    handlers: dict = {}

    def handle_data(self, _data: dict):
        t = _data.get('t')
        op = _data.get('op')
        s = _data.get('s')
        data = _data.get('d')
        log.debug(f"[socket][received]: t: {t} op: {op} s: {s}")
        if t:
            self.call(data, t)

    def call(self, data: dict, type: str):
        if type in self.handlers or EventType.ANY in self.handlers:
            match type:
                case EventType.READY:
                    data = AccountInfo(data)
                case EventType.MESSAGE_CREATE:
                    data = Message(data)
                case EventType.MESSAGE_UPDATE:
                    data = Message(data)
                case _:
                    data = Event(data, type)
            for i in (EventType.ANY, type):
                if i not in self.handlers:
                    continue
                for func in self.handlers[i]:
                    try:
                        if i == EventType.ANY:
                            func(data, type)
                        else:
                            func(data)
                    except Exception as e:
                        log.error(f"[event][{func}]Error: {e}{'' if not self.detailed_error else f'\n{format_exc()}'}")

    def event(self, type: str):
        """
        Decorator for registering an event handler.

        :param type: Event type string or udiscord.EventType.ANY.
        :return: Decorator function.
        """
        def registerHandler(handler):
            self.add_handler(type, handler)
            return handler
        return registerHandler

    def add_handler(self, type: str, handler):
        """
        Registers an event handler for a specific event type.

        :param type: Event type string or udiscord.EventType.ANY.
        :param handler: Function to handle the event.
        :return: The registered handler.
        """
        if type in self.handlers:
            self.handlers[type].append(handler)
        else:
            self.handlers[type] = [handler]
        return handler

    @staticmethod
    def command_validator(commands: list, handler):
        def wrapped_handler(data):
            if not isinstance(data.content, str):
                return
            
            message_content = data.content.lower()
            for command in commands:
                if message_content.startswith(command.lower()):
                    data.content = data.content[len(command):].strip()
                    handler(data)
                    break
        return wrapped_handler

    def command(self, commands: list):
        """
        Decorator for registering a command handler.

        :param commands: List of commands.
        :return: Decorator function.
        """
        def registerCommands(handler):
            self.add_command(commands, handler)
            return handler
        return registerCommands

    def add_command(self, commands: list, handler):
        """
        Registers a command handler for processing messages.

        :param commands: List of commands.
        :param handler: Function to execute when a command is detected.
        :return: Command validator function.
        """
        if EventType.MESSAGE_CREATE in self.handlers:
            self.handlers[EventType.MESSAGE_CREATE].append(self.command_validator(commands, handler))
        else:
            self.handlers[EventType.MESSAGE_CREATE] = [self.command_validator(commands, handler)]
        return self.command_validator



import logging
from datetime import datetime
from colorama import Fore, Style, init


init(autoreset=True)

class Logger:
    """
    A customizable logging class with colorized output.

    Supports different logging levels with color-coded messages 
    for improved readability in the console.
    """

    def __init__(self, level=logging.INFO):
        """
        Initializes the logger with a specified logging level.

        :param level: The minimum logging level to display (default: INFO).
        """
        self.logger = logging.getLogger("Logger")
        self.logger.setLevel(level)
        
        self.console_handler = logging.StreamHandler()

        self.COLORS = {
            logging.DEBUG: Fore.GREEN,
            logging.INFO: Fore.CYAN,
            logging.WARNING: Fore.YELLOW,
            logging.ERROR: Fore.RED,
            logging.CRITICAL: Fore.RED
        }

        self.LEVEL_COLORS = {
            logging.DEBUG: Fore.GREEN + Style.BRIGHT,
            logging.INFO: Fore.CYAN + Style.BRIGHT,
            logging.WARNING: Fore.YELLOW + Style.BRIGHT,
            logging.ERROR: Fore.RED + Style.BRIGHT,
            logging.CRITICAL: Fore.RED + Style.BRIGHT
        }
        self.DATE_COLOR = Fore.LIGHTBLACK_EX

        formatter = logging.Formatter(fmt="%(message)s")

        self.console_handler.setFormatter(formatter)
        self.logger.addHandler(self.console_handler)

    def _colorize(self, message, level):

        level_name = logging.getLevelName(level)
        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        colored_timestamp = self.DATE_COLOR + timestamp + Style.RESET_ALL
        colored_level = self.LEVEL_COLORS.get(level, Fore.WHITE) + level_name + Style.RESET_ALL
        colored_message = self.COLORS.get(level, Fore.WHITE) + message + Style.RESET_ALL

        return f"{colored_timestamp} - {colored_level} - {colored_message}"

    def info(self, message):
        """
        Logs an informational message.

        :param message: The message to log.
        """
        colored_message = self._colorize(message, logging.INFO)
        self.logger.info(colored_message)

    def warning(self, message):
        """
        Logs a warning message.

        :param message: The message to log.
        """
        colored_message = self._colorize(message, logging.WARNING)
        self.logger.warning(colored_message)

    def error(self, message):
        """
        Logs an error message.

        :param message: The message to log.
        """
        colored_message = self._colorize(message, logging.ERROR)
        self.logger.error(colored_message)

    def debug(self, message):
        """
        Logs a debug message.

        :param message: The message to log.
        """
        colored_message = self._colorize(message, logging.DEBUG)
        self.logger.debug(colored_message)

    def critical(self, message):
        """
        Logs a critical error message.

        :param message: The message to log.
        """
        colored_message = self._colorize(message, logging.CRITICAL)
        self.logger.critical(colored_message)

    def set_level(self, level):
        """
        Sets the logging level.

        :param level: The new logging level (e.g., logging.DEBUG, logging.ERROR).
        """
        self.logger.setLevel(level)



from setuptools import setup, find_packages

with open("README.md", "r") as file:
	long_description = file.read()

link = 'https://github.com/alx0rr/discord/archive/refs/heads/main.zip'
ver = '1.3.8'

setup(
	name = "udiscord",
	version = ver,
	url = "https://github.com/alx0rr/discord",
	download_url = link,
	license = "MIT",
	author = "alx0rr",
	author_email = "anon.mail.al@proton.me",
	description = "Library for creating discord bots and scripts.",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	keywords = [
		"discord.py",
		"discord",
		"discord-py",
		"discord-bot",
		"api",
		"python",
		"python3",
		"python3.x",
		"alx0rr",
		"official",
		"sync",
		"async",
		"udiscord",
		"discord-self-bot"
	],
	install_requires = [
		"requests",
		"ujson",
		"logging",
		"websocket-client",
		"colorama",
		"aiohttp"
	],
	packages = find_packages()
)




class EventType:
    """
    A class containing constant event types for use with event handling.

    These event types represent various events that can be received or handled
    from the Discord WebSocket API. They are used to identify the type of event
    that is triggered, such as user updates, messages, interactions, and more.
    """
    ANY = "ANY"
    READY="READY"
    PRESENCE_UPDATE="PRESENCE_UPDATE"
    MESSAGE_CREATE="MESSAGE_CREATE"
    MESSAGE_UPDATE="MESSAGE_UPDATE"
    MESSAGE_DELETE="MESSAGE_DELETE"
    MESSAGE_ACK="MESSAGE_ACK"
    INTERACTION_CREATE="INTERACTION_CREATE"
    USER_SETTINGS_PROTO_UPDATE="USER_SETTINGS_PROTO_UPDATE"
    SESSIONS_REPLACE="SESSIONS_REPLACE"
    PRESENCE_UPDATE="PRESENCE_UPDATE"
    GUILD_AUDIT_LOG_ENTRY_CREATE="GUILD_AUDIT_LOG_ENTRY_CREATE"
    FRIEND_REQUEST="RELATIONSHIP_ADD"
    DELETE_FRIEND="RELATIONSHIP_REMOVE"
    USER_NON_CHANNEL_ACK="USER_NON_CHANNEL_ACK"
    CALL_CREATE="CALL_CREATE"
    CALL_UPDATE="CALL_UPDATE"
    VOICE_STATE_UPDATE="VOICE_STATE_UPDATE"
    CALL_UPDATE="CALL_UPDATE"
    NOTIFICATION_CENTER_ITEMS_ACK="NOTIFICATION_CENTER_ITEMS_ACK"
    TYPING_START="TYPING_START"
    CONTENT_INVENTORY_INBOX_STALE="CONTENT_INVENTORY_INBOX_STALE"
    CHANNEL_UPDATE="CHANNEL_UPDATE"


from . import UserProfile, Guild, Channel

class Invites:
    def __init__(self, data: list):
        self.data: dict = data
        self.size: int = len(data)
        self.invites: list[Invite] = list()
        for i in data:
            self.invites.append(
                Invite(i)
            )



class Invite:
    def __init__(self, data: dict):
        self.data: dict = data
        self.type: int = data.get('type')
        self.code: str = data.get('code')
        self.inviter: UserProfile = UserProfile(data.get("inviter", {}))
        self.max_age: int = data.get("max_age")
        self.created_at: str = data.get("created_at")
        self.expires_at = data.get("expires_at")
        self.flags: int = data.get("flags")
        self.guild: Guild = Guild(data.get("guild", {}))
        self.guild_id: str = data.get("guild_id")
        self.channel: Channel = Channel(data.get("channel", {}))
        self.uses: int = data.get("uses")
        self.max_uses: int = data.get("max_uses")
        self.temporary: bool = data.get("temporary")
        self.approximate_member_count: int = data.get("approximate_member_count")
        self.approximate_presence_count: int = data.get("approximate_presence_count")




class AuditLog:
	def __init__(self, data: dict):
		self.data = data
		self.audit_log_entries = data.get("audit_log_entries", [])
		self.users = data.get("users", [])
		self.integrations = data.get("integrations", [])
		self.webhooks = data.get("webhooks", [])
		self.guild_scheduled_events = data.get("guild_scheduled_events", [])
		self.threads = data.get("threads", [])
		self.application_commands = data.get("application_commands", [])
		self.auto_moderation_rules = data.get("auto_moderation_rules", [])



from .exceptions import *
from . import log

from requests import Session, Response
from aiohttp import ClientSession, ClientResponse
from json import dumps

class Requester:
	def __init__(self, user_agent: str):
		self.web_api = "https://discord.com/api/v9"
		self.session = Session()
		self.user_agent = user_agent
		self.token = None
		self.userId = None

	def headers(self, headers: dict = None) -> dict:
		default_headers = {
			"content-type": "application/json",
			"user-agent": self.user_agent,
		}
		if self.token:
			default_headers["authorization"] = self.token
		if headers: default_headers.update(headers)
		return default_headers



	def make_request(self, method: str, endpoint: str = None, body: dict | bytes = None, allowed_code: int = 200, proxies: dict = None, headers: dict = None, api: str = None) -> Response:
		response = self.session.request(method, f"{api or self.web_api}{endpoint or ''}", data=dumps(body) if isinstance(body, dict) else body if body is not None else None, headers=self.headers(headers), proxies=proxies)
		log.debug(f"[https][{method}][{api or ''}{endpoint or ''}][{response.status_code}]: {len(body) if isinstance(body, bytes) else body}")
		return checkException(response.text) if response.status_code != allowed_code else response


	async def make_async_request(self, method: str, endpoint: str = None, body: dict | bytes = None, allowed_code: int = 200, headers: dict = None , api: str = None) -> ClientResponse:
		async with ClientSession() as asyncSession:
			response = await asyncSession.request(method, f"{api or self.web_api}{endpoint or ''}", data=dumps(body) if isinstance(body, dict) else body if body is not None else None, headers=self.headers(headers))
			log.debug(f"[https][{method}][{endpoint or ''}][{response.status}]: {len(body) if isinstance(body, bytes) else body}")
			return checkException(await response.text()) if response.status != allowed_code else response